/* mac2unix.c -  Version 1.0 */

/*----------------------------------------------------------------------------*/
/*
 * This program converts text files from Macintosh format [CR] into
 * UNIX format [LF].
 *
 * This program orginally written for Linux 2.2.x using Glibc 2.0.7
 *
 * Copyright (c) 1999 Kevin Holbrook. All rights reserved.
 * 
 * Problems and corrections can be sent to the author at the 
 * following email address: KevinHolbrook@yahoo.com
 *
 *
 * This source is distributed on an 'AS IS' basis, without warranty of any kind,
 * either express or implied, and I hereby disclaim all such warranties,
 * including without limitation, any warranties of merchantability,
 * fitness for a particular purpose or non-infringement.
 *
 * Permission is granted to use, copy, alter, and re-distribute this source
 * provided the original copyright and disclaimer remains intact.
 */
/*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/param.h>   /* this indirectly resolves PATH_MAX on Linux */


#define BUFFER_SIZE   1028
#define READ_SIZE     (BUFFER_SIZE - 1)

static char  buffer[BUFFER_SIZE];



static void usage(void)
{
  printf("  mac2unix  <input filename>  [output filename]\n\n");
  printf("\tConverts Macintosh delimited text files to UNIX text format.\n");
  printf("\tIf output filename is omitted, the input file is safely overwritten.\n\n");
}


int main(int argc, void** argv)
{
  char*   input_name;
  char*   output_name;
  FILE*   input_file;
  FILE*   output_file;
  size_t  num_read;
  int     loop;
  int     status;
  int     overwrite_flag = 0;
  char    new_name[PATH_MAX];


  /* make sure we have enough arguments, not that we like arguing... */
  if ((argc < 2) || (argc > 3))
  {
    usage();
    return(EXIT_FAILURE);
  }

  input_name = argv[1];

  if (argc == 3)
  {
    output_name = argv[2];

    if (strcmp(input_name, output_name) == 0)
      overwrite_flag = 1;
  }
  else /* output file was omitted, flag overwriting */
  {
    output_name = input_name;
    overwrite_flag = 1;
  }

  if (overwrite_flag)
  {
    memset(new_name, 0, PATH_MAX);

    /* if name is too long to append an extension, use temporary name */
    if (strlen(input_name) >= PATH_MAX-4)
      strcpy(new_name, "temp01234");
    else
    {
      strcpy(new_name, input_name);
      strcat(new_name, ".tmp");
    }

    status = rename(input_name, new_name);

    if (status == -1)
    {
      printf("Unable to rename file %s during conversion.\n", input_name);
    }

    input_name = new_name;
  }

  input_file = fopen(input_name, "rb");

  if (input_file == NULL)
  {
    printf("Unable to open input file.\n");

    /* restore the original file name */
    status = rename(new_name, input_name);
 
    return(EXIT_FAILURE);
  }

  output_file = fopen(output_name, "w");

  if (output_file == NULL)
  {
    fclose(input_file);
    printf("Unable to open output file.\n");

    /* restore the original file name */
    status = rename(new_name, input_name);

    return(EXIT_FAILURE);
  }

  /* clear out the buffer */
  memset(buffer, 0, BUFFER_SIZE);

  /* read in a chunk of text to process */
  num_read = fread(buffer, sizeof(char), READ_SIZE, input_file);

  while (num_read && !ferror(input_file))
  {
    for (loop = 0; loop < num_read; loop++)
    {
      if ( isprint(buffer[loop]) )
        continue;

      /* check for CR and replace with LF */
      if ( buffer[loop] == 13 )
        buffer[loop] = 10;
    }

    fwrite(buffer, sizeof(char), num_read, output_file);

    if ( !feof(input_file) )
    {
      /* clear buffer for next read */
      memset(buffer, 0, BUFFER_SIZE);

      /* get next chunk of text */
      num_read = fread(buffer, sizeof(char), READ_SIZE, input_file);
    }
    else
      num_read = 0;
  } /* while */

  /* close the files */
  fclose(input_file);
  fclose(output_file);

  /* get rid of temporary file */
  status = unlink(new_name);

  if (status == -1)
  {
    printf("Unable to remove temporary file %s\n", new_name);
    return(EXIT_FAILURE);
  }

  return(EXIT_SUCCESS);
}
